@@ -8,7 +8,7 @@ import Analyzer from './analyser'
 import * as Builtins from './builtins'
 import * as config from './config'
 import Executables from './executables'
-import Linter from './linter'
+import { getLinterExecutablePath, Linter } from './linter'
 import { initializeParser } from './parser'
 import * as ReservedWords from './reservedWords'
 import { BashCompletionItem, CompletionItemDataType } from './types'
@@ -41,8 +41,9 @@ export default class BashServer {
     return Promise.all([
       Executables.fromPath(PATH),
       Analyzer.fromRoot({ connection, rootPath, parser }),
-      new Linter({ executablePath: config.getShellcheckPath() }),
-    ]).then(([executables, analyzer, linter]) => {
+      getLinterExecutablePath(),
+    ]).then(([executables, analyzer, linterExecutablePath]) => {
+      const linter = new Linter({ executablePath: linterExecutablePath })
       return new BashServer(connection, executables, analyzer, linter, capabilities)
     })
   }
@@ -231,22 +232,17 @@ export default class BashServer {
 
     const explainshellEndpoint = config.getExplainshellEndpoint()
     if (explainshellEndpoint) {
-      this.connection.console.log(`Query ${explainshellEndpoint}`)
       try {
-        const response = await this.analyzer.getExplainshellDocumentation({
+        const { helpHTML } = await this.analyzer.getExplainshellDocumentation({
           params,
           endpoint: explainshellEndpoint,
         })
 
-        if (response.status === 'error') {
-          this.connection.console.log(
-            `getExplainshellDocumentation returned: ${JSON.stringify(response, null, 4)}`,
-          )
-        } else {
+        if (helpHTML) {
           return {
             contents: {
               kind: 'markdown',
-              value: new TurndownService().turndown(response.helpHTML),
+              value: new TurndownService().turndown(helpHTML),
             },
           }
         }
